const express = require('express');
const http = require('http');
const cors = require('cors');
const socketIo = require('socket.io');
const mongoose = require('mongoose');

const app = express();
const server = http.createServer(app);
const io = socketIo(server);
app.use(cors()); // Enable CORS

const url = "mongodb://wallychapin_test:Enero2024+@mongo.db.mdbgo.com:8604/wallychapin_pruebas"
mongoose.connect(url, { useNewUrlParser: true, useUnifiedTopology: true });

const UserSchema = new mongoose.Schema({
    rol: Number,
    name: String,
    email: String,
    credits: Number,
    user: String,
    password: String
});
const User = mongoose.model('User', UserSchema);


const GameSchema = new mongoose.Schema({
    groupId: String,
    result: String,
    client: String,
    clientGuess: String,
    status: String,
    date: { type: Date, default: Date.now }
});
const GameResult = mongoose.model('Game', GameSchema);

let credits = 15;
let onlinePlayers = 0;
let users = [];
let gamers = [];

const groups = [];
const groupSize = 2;
let countdownInterval = null;
let countdownTime = 60; // Countdown in seconds


io.on('connect', (socket) => {
    console.log('a user connected');

    let assignedGroup = null;
    let userId = generateRandomValue(8);
    users.push(userId);

    // Find an available group with less than 2 players
    for (const group of groups) {
        if (group.length < groupSize) {
            assignedGroup = group;
            break;
        }
    }

    // If no group is found, create a new one
    if (!assignedGroup) {
        assignedGroup = [];
        groups.push(assignedGroup);
    }

    assignedGroup.push(socket.id);
    socket.group = assignedGroup;

    const groupId = `group-${groups.indexOf(assignedGroup)}`;
    const credits = 15;

    console.log(`User ${userId} joined ${groupId}`);
    socket.emit('info', { userId, groupId, credits });

    onlinePlayers++;
    io.emit('onlinePlayers', { count: onlinePlayers });


    // Notify group members when a group is full
    if (assignedGroup.length === groupSize) {
        assignedGroup.forEach(socketId => {
            io.to(socketId).emit('groupReady', { groupId });
        });
    }

    // Start countdown when the first group is full
    if (!countdownInterval) {
        startCountdown();
    }

    socket.on('guess', (userData) => {

        console.log(userData);
        const group = socket.group;
        if (group) {
            gamers.push(userData);
            socket.userData = userData; // Store user data in the socket object
        }
    });

    socket.on('disconnect', () => {
        console.log('user disconnected');
        const group = socket.group;
        if (group) {
            const index = group.indexOf(socket.id);
            if (index !== -1) group.splice(index, 1);
            if (group.length === 0) {
                const groupIndex = groups.indexOf(group);
                if (groupIndex !== -1) groups.splice(groupIndex, 1);
            }
        }

        let index = users.findIndex(a => a.userId == userId);
        users.splice(index, 1);

        console.log('user disconnected');
        onlinePlayers--;
        io.emit('onlinePlayers', { count: onlinePlayers });
    });

});

const startCountdown = () => {
    countdownTime = 15; // Reset countdown to 60 seconds
    countdownInterval = setInterval(() => {
        if (countdownTime <= 0) {
            clearInterval(countdownInterval);
            countdownInterval = null;
            io.emit('countdown', { time: 0 });

            //send results to all grups
            groups.forEach((group, index) => {

                const result = generateResult(); // Implement your result generation logic

                group.forEach(async (socketId) => {

                    const clientSocket = io.sockets.sockets.get(socketId);
                    const userData = clientSocket.userData || {};

                    //only gamers playing
                    if (userData != {}) {
                        const status = userData.value == result ? 'won' : 'lose';
                        const credits = userData.credits--;

                        io.to(socketId).emit('result', { groupId: group, result, status, credits });

                        await GameResult.create({
                            groupId: userData.groupId,
                            result: result,
                            client: userData.userId,
                            clientGuess: userData.value,
                            status: status
                        });

                        clientSocket.userData = {};
                    }


                    //const status = userData.value == result ? 'won' : 'lose';


                    //console.log(status);
                    //console.log(credits);

                    //io.to(socketId).emit('status', {  credits });

                    /*
                    let currentGamers = gamers.filter(a => a.groupId == group);

                    currentGamers.forEach(async (userData) => {

                        const status = userData.value == result ? 'won' : 'lose';
                        const credits = userData.credits--;

                        console.log(status);
                        console.log(credits);

                        io.to(socketId).emit('status', { status, credits });


                        await GameResult.create({
                            groupId: userData.groupId,
                            result: result,
                            client: userData.userId,
                            clientGuess: userData.value,
                            status: status
                        });


                    });
                    */

                    // const clientSocket = io.sockets.sockets.get(socketId);
                    // const userData = clientSocket ? clientSocket.userId : 'unknown'; // Replace with actual user ID logic
                    // console.log(userData);

                    //const clientSocket = io.sockets.sockets.get(socketId);
                    //console.log(clientSocket);


                    // Assuming userData was set on the socket previously
                    //let userData = io.sockets.sockets.client; //clientSocket ? clientSocket.userData : { id: 'unknown' }; // Use default value if not set

                    // Log the userData or handle it as needed
                    //console.log('Client user data:', userData);

                    /*
                    //const clientSocket = io.sockets.sockets.get(socketId);
                    //const userData = clientSocket ? clientSocket.userId : 'unknown'; // Replace with actual user ID logic
                    const status = userData.value == result ? 'won' : 'lose';
                    const credits = userData.credits--;

                    io.to(socketId).emit('status', { status, credits });

                    await GameResult.create({
                        groupId: userData.groupId,
                        result: result,
                        client: userData.userId,
                        clientGuess: userData.value,
                        status: status
                    });
                    */


                });


                /*
                // group.forEach(socketId => {
                //     io.to(socketId).emit('result', data);
                // });

                const clientSocket = io.sockets.sockets.get(socketId);
                const userData = clientSocket ? clientSocket.userId : 'unknown'; // Replace with actual user ID logic
                const status = userData.value == result ? 'won' : 'lose';
                const credits = userData.credits--;

                // Emit the status to all group members
                group.forEach(socketId => {
                    io.to(socketId).emit('status', { status, credits });
                });

                // Save the result to MongoDB for each client
                group.forEach(async () => {

                    //const clientSocket = io.sockets.sockets.get(socketId);
                    //const userData = clientSocket ? clientSocket.userId : 'unknown'; // Replace with actual user ID logic
                    await GameResult.create({
                        groupId: userData.groupId,
                        result: result,
                        client: userData.userId,
                        clientGuess: userData.value,
                        status: status
                    });
                });
                */

            });


            // Delay of 10 seconds before restarting countdown
            setTimeout(() => {
                startCountdown();
            }, 10000); // 10 seconds delay


            // You may want to handle game end logic here
        } else {
            io.emit('countdown', { time: countdownTime });
            countdownTime--;
        }
    }, 1000); // Emit countdown every second
};


/*
// Send results to each group every minute
setInterval(() => {
    groups.forEach((group, index) => {
        if (group.length === groupSize) {
            const result = generateResult(); // Implement your result generation logic
            group.forEach(socketId => {
                io.to(socketId).emit('result', { groupId: group, result });
            });
        }
    });
}, 30000);
*/


/*
let onlinePlayers = 0;
let users = [];
 
const groups = {};
const groupSize = 2;
 
io.on('connect', (socket) => {
 
    let assignedGroup = null;
 
    // Find an available group with less than 2 players
    for (const [groupId, group] of Object.entries(groups)) {
        if (group.length < groupSize) {
            assignedGroup = groupId;
            break;
        }
    }
 
    // If no group is found, create a new one
    if (!assignedGroup) {
        assignedGroup = `group_${Object.keys(groups).length + 1}`;
        groups[assignedGroup] = [];
    }
 
    groups[assignedGroup].push(socket.id);
    socket.join(assignedGroup);
    socket.groupId = assignedGroup;
 
 
    let userId = generateRandomValue(8);
    console.log("userId: " + userId);
    users.push(userId);
    //io.emit('userId', userId);
 
    //socket.emit('userId', userId);
    socket.emit('info', { userId, groupId: assignedGroup });
 
    console.log('a user connected');
    onlinePlayers++;
    io.emit('onlinePlayers', { count: onlinePlayers });
 
    console.log(`User ${userId} joined ${assignedGroup}`);
 
    // Notify group members when a group is full
    if (groups[assignedGroup].length === groupSize) {
        io.to(assignedGroup).emit('groupReady', { groupId: assignedGroup });
    }
 
 
    socket.on('guess', (data) => {
        console.log(data);
        // Handle the guess and update user credits
        // Broadcast the result every 30 seconds
    });
 
    socket.on('disconnect', (userId) => {
 
        let index = users.findIndex(a => a.userId == userId);
        users.splice(index, 1);
 
        console.log('user disconnected');
        onlinePlayers--;
        io.emit('onlinePlayers', { count: onlinePlayers });
    });
});
 
setInterval(() => {
    const result = `${Math.floor(Math.random() * 100)}${String.fromCharCode(65 + Math.floor(Math.random() * 26))}`;
    io.emit('result', { result });
}, 30000);
*/

server.listen(3000, () => {
    console.log('Server is running on port 3000');
});

app.get('/', (req, res) => {
    res.json({ message: "conected to game api" });
});


const generateResult = () => {
    const num1 = Math.floor(Math.random() * 10); // Generates a random integer between 0 and 9
    const num2 = Math.floor(Math.random() * 10); // Generates a random integer between 0 and 9
    const letter = String.fromCharCode(65 + Math.floor(Math.random() * 26)); // Generates a random uppercase letter

    return num1.toString() + num2.toString() + letter;
};


const generateRandomValue = (j) => {
    const characters = '1234567890abcdefghijklmnopqrstuvwxyz';
    let result = '';
    for (let i = 0; i < j; i++) {
        result += characters[Math.floor(Math.random() * characters.length)];
    }
    return result;
}
